/******************************************************************************
 * FILE: L_avionics.c
 *
 * DESCRIPTION:
 * function 'main()' is here. Multiple threads are created for various estimation
 * and control functions onboard the UAV. POSIX threads API is used for thread
 * creation, synchronizition and priority definition.
 *
 *
 * University of Minnesota 
 * Aerospace Engineering and Mechanics 
 * Copyright 2011 Regents of the University of Minnesota. All rights reserved.
 *
 * $Id: L_avionics.c 332 2011-04-14 18:43:14Z murch $
 ******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <termios.h>
#include <math.h>
#include <pthread.h>
#include <sched.h>
#include <cyg/posix/pthread.h>
#include <cyg/kernel/kapi.h>
#include <cyg/posix/time.h>
#include <cyg/posix/signal.h>
#include <cyg/cpuload/cpuload.h>
#include <cyg/gpt/mpc5xxx_gpt.h>
#include <cyg/io/mpc5xxx_gpio.h>
#include <cyg/io/i2c_mpc5xxx.h>

#include "globaldefs.h"
#include "utils/navfunc.c"
#include "utils/misc.c"
#include "utils/rtalarms.c"
#include "utils/serial_mpc5200.c"

// threads
#include "L_ahrs.c"
#include "L_navmain.c"
#include "control/control_manager.c"
#include "L_dnlink_cas.c"
#include "L_datalogger_ramfs.c"



#if (defined MNAV || defined HILS)
#include "L_imu_daq.c"
#else
#include "L_sensordaq.c"
#endif

#ifdef GPIO_ENABLED
#include "utils/gpio.c"
#endif



//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//pre-defined defintions
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//globals
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
extern double airdata_bias[];

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//thread prototypes
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
extern void *ahrs_main(void *thread_id);
extern void *imugps_acq(void *thread_id);
extern void *navigation(void *thread_id);
extern void *control_uav(void *thread_id);
extern void *downlink(void *thread_id);
extern void *datalogger(void *thread_id);

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// function prototypes
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void AirData_bias_estimate();
char *printEnum( enum errdefs errcode );



int main(int argc, char **argv)
{
	pthread_t 		threads[NUM_THREADS];
	cyg_uint8 		gpioTriggerStatus;
	pthread_attr_t	attr;
	struct 			sched_param		param;
	int 			rc[NUM_THREADS];

#ifdef OFP_BUILD
	word			servo_deflect;
	static double	t0, t1, t_elapsed=0;
	struct 			timespec 	airdata_biasestimate_timer;

	// specify timer
	airdata_biasestimate_timer.tv_sec = 0;
	airdata_biasestimate_timer.tv_nsec= 100000000;
#endif

	cyg_cpuload_calibrate (&cpuCalibrationData);
	cyg_cpuload_create (&cpuload,cpuCalibrationData,&loadhandle);

	//initialize RT wallclock
	get_Time();

#ifdef GPIO_ENABLED
	short toggle=TRUE;
	gpio_init();
#endif

	// initiallize mpc5200 PWM I/O channels
	GPT_Init();

	/* PWM Output */
	if( GPT_Open_PWM(0) < 0) fprintf(stderr, "\n GPT_Open_PWM() failed Ch.0");	// left flap
	if( GPT_Open_PWM(1) < 0) fprintf(stderr, "\n GPT_Open_PWM() failed Ch.1");	// right flap
	if( GPT_Open_PWM(2) < 0) fprintf(stderr, "\n GPT_Open_PWM() failed Ch.2");	// right aileron (aileron Y'd)
	if( GPT_Open_PWM(3) < 0) fprintf(stderr, "\n GPT_Open_PWM() failed Ch.3");	// elevator
	if( GPT_Open_PWM(4) < 0) fprintf(stderr, "\n GPT_Open_PWM() failed Ch.4");	// throttle
	if( GPT_Open_PWM(5) < 0) fprintf(stderr, "\n GPT_Open_PWM() failed Ch.5");	// rudder
	if( GPT_Open_PWM(7) < 0) fprintf(stderr, "\n GPT_Open_PWM() failed Ch.7");	// left aileron

	// clock-alarms init
	rt_alarms_init();

	// center the control surfaces
	GPT_PWM_Write_Width(0, PWM_1ms_COUNTS+PWM_COUNTS_RANGE/2);		// left flap
	GPT_PWM_Write_Width(1, PWM_1ms_COUNTS+PWM_COUNTS_RANGE/2);		// right flap
	GPT_PWM_Write_Width(2, PWM_1ms_COUNTS+PWM_COUNTS_RANGE/2);		// right aileron (aileron Y'd)
	GPT_PWM_Write_Width(3, PWM_1ms_COUNTS+PWM_COUNTS_RANGE/2);		// elevator
	GPT_PWM_Write_Width(4, PWM_1ms_COUNTS);							// throttle
	GPT_PWM_Write_Width(5, PWM_1ms_COUNTS+PWM_COUNTS_RANGE/2);		// rudder
	GPT_PWM_Write_Width(7, PWM_1ms_COUNTS+PWM_COUNTS_RANGE/2);		// left aileron


	/*********************************************************************
	 *initialize mutex and conditional variables
	 *********************************************************************/
	pthread_mutex_init(&mutex_imu,NULL);
	pthread_mutex_init(&mutex_gps,NULL);
	pthread_mutex_init(&mutex_nav,NULL);
	pthread_mutex_init(&mutex_fdat,NULL);
	pthread_mutex_init(&mutex_sPort0,NULL);


	/*********************************************************************
	 * multiple threads creation
	 *********************************************************************/
	/*initialize and set thread detached attribute */
	pthread_attr_init(&attr);
	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
	/*set scheduling policy */
	pthread_attr_setinheritsched(&attr,PTHREAD_EXPLICIT_SCHED);
	pthread_attr_setschedpolicy(&attr, SCHED_FIFO);
	/*set thread priority */

	param.sched_priority =  sched_get_priority_max(SCHED_FIFO) - 3;
	pthread_attr_setschedparam(&attr, &param);
	rc[4] = pthread_create(&threads[4], &attr, downlink, (void *)4);
	strcpy(threadstat[4].name, "DNLINK");
	threadstat[4].priority = param.sched_priority;

	param.sched_priority = sched_get_priority_max(SCHED_FIFO);
	pthread_attr_setschedparam(&attr, &param);
	/// Thread for L_imu_daq or L_sensordaq
	rc[0] = pthread_create(&threads[0], &attr, sensordaq,  (void *)0);
	strcpy(threadstat[0].name, "DAQ");
	threadstat[0].priority = param.sched_priority;
#ifdef  DATALOG_ENABLE
	param.sched_priority = sched_get_priority_max(SCHED_FIFO) - 4;
	pthread_attr_setschedparam(&attr, &param);
	rc[5] = pthread_create(&threads[5], &attr, datalogger, (void *)5);
	strcpy(threadstat[1].name, "DATALOG");
	threadstat[5].priority = param.sched_priority;
#endif
	// note ahrs init early!

	param.sched_priority = sched_get_priority_max(SCHED_FIFO);
	pthread_attr_setschedparam(&attr, &param);
	rc[1] = pthread_create(&threads[1], &attr, ahrs_main, (void *)1);
	strcpy(threadstat[1].name, "AHRS");
	threadstat[1].priority = param.sched_priority;

	param.sched_priority =  sched_get_priority_max(SCHED_FIFO) - 2;
	pthread_attr_setschedparam(&attr, &param);
	rc[2] = pthread_create(&threads[2], &attr, navigation, (void *)2);
	strcpy(threadstat[2].name, "NAV");
	threadstat[2].priority = param.sched_priority;


#ifdef OFP_BUILD
	// ============================= UAV init ================================
	
	
	
	t0 = get_Time();

	
	while (t_elapsed < 16.0)
	{
		t1 = get_Time();
		t_elapsed = t1-t0;
		
		// Take zero on pressure sensors or micronav if in use.
		#ifndef MB_ADS
			if (t_elapsed >= 4.0 && t_elapsed < 10.0){AirData_bias_estimate();}
		#endif
		
		// move the servos to see if all are functioning
		if (t_elapsed >= 10.0 && t_elapsed < 14.0)
		{
			servo_deflect = (word)( PWM_1ms_COUNTS + PWM_COUNTS_RANGE * (1 + sin(half_pi*t_elapsed)) / 2.0 );		// cycle time period = 4s

			GPT_PWM_Write_Width (0, servo_deflect);		
			GPT_PWM_Write_Width (1, servo_deflect);		
			GPT_PWM_Write_Width (2, servo_deflect);				
			GPT_PWM_Write_Width (3, servo_deflect);		
			//GPT_PWM_Write_Width (4, servo_deflect);		// throttle
			GPT_PWM_Write_Width (5, servo_deflect);	
			GPT_PWM_Write_Width (7, servo_deflect);		
		}

		nanosleep(&airdata_biasestimate_timer, NULL);
	}
		

	
	// center the control surfaces
	GPT_PWM_Write_Width(0, PWM_1ms_COUNTS+PWM_COUNTS_RANGE/2);		// left flap
	GPT_PWM_Write_Width(1, PWM_1ms_COUNTS+PWM_COUNTS_RANGE/2);		// right flap
	GPT_PWM_Write_Width(2, PWM_1ms_COUNTS+PWM_COUNTS_RANGE/2);		// right aileron (aileron Y'd)
	GPT_PWM_Write_Width(3, PWM_1ms_COUNTS+PWM_COUNTS_RANGE/2);		// elevator
	GPT_PWM_Write_Width(4, PWM_1ms_COUNTS);							// throttle
	GPT_PWM_Write_Width(5, PWM_1ms_COUNTS+PWM_COUNTS_RANGE/2);		// rudder
	GPT_PWM_Write_Width(7, PWM_1ms_COUNTS+PWM_COUNTS_RANGE/2);		// left aileron	
	// =====================================================================================
#endif


	param.sched_priority = sched_get_priority_max(SCHED_FIFO) - 1;
	pthread_attr_setschedparam(&attr, &param);
	rc[3] = pthread_create(&threads[3], &attr, control_uav, (void *)3);
	strcpy(threadstat[3].name, "CLAW");
	threadstat[3].priority = param.sched_priority;


	//++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	//main-loop
	//++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	while( ofpMode != terminate )
	{

#ifdef GPIO_ENABLED
		// check if trigger has arrived for data transfer / OFP termination
		gpioTriggerStatus = GPIO_GPW_GetInputStatus( MPC5XXX_GPW_GPIO_WKUP_7 );
		if ( gpioTriggerStatus == 0 ){
			//		if (( get_Time() > 1500 )){

			ofpMode = terminate;
		}

		GPIO_GPW_Output( MPC5XXX_GPS_IRDA_TX, toggle );
		toggle = !toggle;
#endif
//		fprintf( stderr, "\n data: ax=%.2f, ay=%.2f,az=%.2f,\n p=%.2f, q=%.2f,r=%.2f,\n	hx=%.2f, hy=%.2f, hz=%.2f,\n Vs=%.2f, lat=%f, alt=%f, ias=%f,\nGPS_TOW=%d, GPS_status=%s",\
		imupacket.ax,imupacket.ay,imupacket.az,\
		imupacket.p,imupacket.q,imupacket.r,\
		imupacket.hx,imupacket.hy,imupacket.hz,\
		imupacket.Vs,gpspacket.lat, imupacket.h,\
		imupacket.ias, gpspacket.GPS_TOW, printEnum(gpspacket.err_type) );


		sched_yield();
		sleep(1);
	} //end while


	// external trigger received. transfer file to host pc, and terminate OFP
	diag_printf("\n Initiating filetransfer over TFTP...\n");
	pthread_join( threads[5], NULL );	//DATALOGGER
	pthread_join( threads[0], NULL );	//DAQ
	pthread_join( threads[3], NULL );	//CLAW
	pthread_join( threads[1], NULL );	//AHRS
	pthread_join( threads[2], NULL );	//NAV
	pthread_join( threads[4], NULL );	//DNLINK
	diag_printf("\n Threads terminated.");

#ifdef  DATALOG_ENABLE
	filetransferToGCS( getfilename, putfilename );
	sleep(1);
	umountFilesystem();
#endif
	close(sPort0);	close(sPort3);	// ports are opened by DNLINK thread


	/**********************************************************************
	 * close
	 **********************************************************************/
	// close PWM channels
	GPT_PWM_Exit(2);GPT_PWM_Exit(3);GPT_PWM_Exit(4);GPT_PWM_Exit(5);
	GPT_PWM_Exit(0);GPT_PWM_Exit(1);GPT_PWM_Exit(7);

	pthread_mutex_destroy(&mutex_imu);
	pthread_mutex_destroy(&mutex_gps);
	pthread_mutex_destroy(&mutex_nav);
	pthread_mutex_destroy(&mutex_fdat);
	pthread_mutex_destroy(&mutex_sPort0);

	rt_alarms_destroy();

	pthread_attr_destroy(&attr);
	pthread_exit(NULL);

	return 0;
}  // End of main loop




// ==================================== functions ======================================

void AirData_bias_estimate()
{
	static int count;
	static double airdata_bias_prev[2];
	extern double airdata_bias[];

	count++;
	airdata_bias[0] = airdata_bias_prev[0]*(1-1/count) + imupacket.h*(1/count);
	airdata_bias[1] = airdata_bias_prev[1]*(1-1/count) + imupacket.Pd*(1/count);

	airdata_bias_prev[0] = airdata_bias[0]; airdata_bias_prev[1] = airdata_bias[1];
}

char *printEnum( enum errdefs errcode )
{
	switch( errcode ){
	case got_invalid:
		return("got_invalid");

	case checksum_err:
		return("checksum error");

	case gps_nolock:
		return("gps_nolock");

	case data_valid:
		return("data_valid");

	case mag_CalibrationBad:
		return("Magnet. Calibration Bad");

	default:
		return("Unknown error");
	}
}
