/******************************************************************************
 * FILE: L_dnlink.c
 * DESCRIPTION:
 * Sends telemetry data thru serial port, logs binary flight data onboard
 * MPC5200 flash memory.
 *
 *
 * AUTHOR: Rohit Pandita
 *
 * University of Minnesota 
 * Aerospace Engineering and Mechanics 
 * Copyright 2011 Regents of the University of Minnesota. All rights reserved.
 *
 * $Id: L_dnlink_cas.c 332 2011-04-14 18:43:14Z murch $
 ******************************************************************************/
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <errno.h>
#include <string.h>
#include <dirent.h>
#include <stdio.h>
#include <stdlib.h>



#define TELE_PACKET_SIZE  51



/* telemetry packet structure = [ <UUT> time, <empty>, <empty>, p, q, r, alt, IAS, psi, theta, phi, rud, ele, thr, ail, cpuload, lon, lat, statusFlags, <empty> <16bit_CKSUM> ] */

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//function prototypes
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
unsigned short do_chksum(byte* buffer, int packet_len);
unsigned short getStatusFlags();


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//globals
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
extern word delr,dele,delthr;		// control surface demands
extern short whichmode[];			// whichmode: {AHD, IHD, WPN}
extern int sPort0;
extern int sPort3;



void *downlink(void *thread_id)
{
	int bytes=0, rc;
	unsigned long tmp;
	word tele_data[23], output_CKSUM=0;
	static byte sendpacket[TELE_PACKET_SIZE]={'U','U','T',};
	double toc;
	static double tic;

	// Open '/dev/termios0' serial port on mpc5200 for telemetry
	#if  (defined MNAV || defined HILS)
		sPort0 = open_serial(SERIAL_PORT0, BAUDRATE_115200);	//RS232 with micronav avioncs
	#else	
		sPort3 = open_serial(SERIAL_PORT3, BAUDRATE_115200);	//TTL1 with new avionics
	#endif
	
	pthread_mutex_lock(&mutex_sPort0);
	sprintf(&msg[3],"DNLINK thread created");
	write(sPort0, msg, sizeof(msg));
	pthread_mutex_unlock(&mutex_sPort0);


	while( ofpMode != terminate )
	{

		pthread_mutex_lock (&mutex_sPort0);
		rc = pthread_cond_wait (&trigger_downlink, &mutex_sPort0);

		// check if the trigger has arrived
		if (rc == 0) {

			// compute thread jitter
			toc = get_Time();
			threadstat[4].jitter_nsec = ( (toc-tic) - 1/(double)DNLINK_Hz ) * 1E09;
			tic = toc;

			pthread_mutex_lock(&mutex_gps);
			pthread_mutex_lock(&mutex_imu);


			// get current cpu load
			cyg_cpuload_get (loadhandle, &last100ms, &last1s, &last10s);

			// converting imu data to ascii string for transmission over serial port
			tmp = (unsigned long)( get_Time()*1e04 );		// time buffer will now overflow after 59.6 hrs (thats what 4 bytes' worth!)
			memcpy(&tele_data[0],&tmp,4);

			tele_data[2] = (word)(0);
			tele_data[3] = (word)(0);

			tele_data[4] = (word)( imupacket.p*R2D / 200.0 * 0x7FFF );	//rate = 200 deg/s max saturation
			tele_data[5] = (word)( imupacket.q*R2D / 200.0 * 0x7FFF );
			tele_data[6] = (word)( imupacket.r*R2D / 200.0 * 0x7FFF );
			
			tele_data[7] = (word)( (imupacket.h) / 10000.0 * 0x7FFF );			// max Pressure Alt.(m) = 10000 m
			tele_data[8] = (word)( (imupacket.ias) / 80.0 * 0x7FFF );				// max Equivalent Airspeed(EAS) = 80 m/s	

			tele_data[9] = (word)( imupacket.psi*R2D / 180.0 * 0x7FFF );		// Euler angles [psi,theta,phi] (deg)
			tele_data[10]= (word)( imupacket.the*R2D / 90.0 * 0x7FFF );
			tele_data[11]= (word)( imupacket.phi*R2D / 180.0 * 0x7FFF );

			tele_data[12]= (word)( controlpacket.da / AILERON_MAX * 0x7FFF );		// control demands (normalized 0-1)
			tele_data[13]= (word)( controlpacket.de / ELEVATOR_MAX * 0x7FFF );
			tele_data[14]= (word)( controlpacket.dthr * 0x7FFF );
			tele_data[15]= (word)( controlpacket.dr / RUDDER_MAX * 0x7FFF );


			tele_data[16] = last1s;		/* cpuload */

			tmp = (unsigned long)( gpspacket.lon *1e07 );
			memcpy(&tele_data[17],&tmp,4);
			tmp = (unsigned long)( gpspacket.lat *1e07 );
			memcpy(&tele_data[19],&tmp,4);

			tele_data[21] = getStatusFlags();
			tele_data[22] = (word)0;

			memcpy(&sendpacket[3],&tele_data,46);
			// compute 16-bit checksum of output data (excluding the header)
			output_CKSUM = do_chksum (sendpacket, TELE_PACKET_SIZE);
			*(word *)&sendpacket[49] = output_CKSUM;

			// send telemetry data packet to serial port
			#if  (defined MNAV || defined HILS)
				while(bytes != TELE_PACKET_SIZE) bytes += write(sPort0, &sendpacket[bytes], TELE_PACKET_SIZE-bytes); bytes=0;
			#else
				while(bytes != TELE_PACKET_SIZE) bytes += write(sPort3, &sendpacket[bytes], TELE_PACKET_SIZE-bytes); bytes=0;
			#endif

			pthread_mutex_unlock(&mutex_imu);
			pthread_mutex_unlock(&mutex_gps);

			pthread_mutex_unlock(&mutex_sPort0);
		}

	}

//	close(sPort0);	// closed in main()
//	close(sPort3);
	pthread_exit(NULL);
}


//------------------------------- functions -----------------------------------

unsigned short do_chksum(byte* buffer, int packet_len)
{
	word i;
	word sum=0;

	for(i=2;i<packet_len-2;i++) sum = sum + buffer[i];

	return  sum;
}

unsigned short getStatusFlags(void)
{
	unsigned short flags=0;
	extern short whichmode[];

	if (ofpMode == standby) flags = flags | 0x01;
	if (whichmode[0] == TRUE) flags = flags | 0x01<<1;		// AHD mode
	if (whichmode[1] == TRUE) flags = flags | 0x01<<2;		// IHD mode
	if (whichmode[2] == TRUE) flags = flags | 0x01<<3;		// WPN mode
	if (whichmode[0]==FALSE && whichmode[1]==FALSE && whichmode[2]==FALSE) flags = flags | 0x01<<4;  // Manual mode
	if (ofpMode == terminate) flags = flags | 0x01<<5;
	if ( (imupacket.err_type != checksum_err) && (imupacket.err_type != got_invalid) ) flags = flags | 0x01<<6;
	if ( (gpspacket.err_type != checksum_err) && (gpspacket.err_type != got_invalid) ) flags = flags | 0x01<<7;
	if (gpspacket.err_type == data_valid) flags = flags | 0x01<<8;

	return flags;
}
