/******************************************************************************
 * FILE: HoneywellAltimeter.c
 *
 * DESCRIPTION:
 * This is an Object Oriented C class that is able to read the data of 
 * ASDX*030*A24R*DO Honeywell absolute pressure sensor. It has two methods
 * readPressure which samples the pressure data, and getPressure which returns the
 * absolute pressure from pressureMin psi to pressureMax psi
 *
 * CREATED: 07-07-10: Kyle Dobitz 
 * University of Minnesota 
 * Aerospace Engineering and Mechanics 
 * Copyright 2011 Regents of the University of Minnesota. All rights reserved.
 *
 * $Id: HoneywellAltimeter.c 332 2011-04-14 18:43:14Z murch $
 ******************************************************************************/

#include <stdlib.h> 
#include <math.h>
#include <cyg/io/i2c_mpc5xxx.h>
#include "HoneywellAltimeter.h" 

struct honeywellaltimeter{
	cyg_i2c_device device;
	double pressureReading;
	double pressureMin;
	double pressureMax;
	double altitudeReading;
};

HoneywellAltimeter new_HoneywellAltimeter(cyg_i2c_device *i2c_device, double pressureMin, double pressureMax){
	
	HoneywellAltimeter this_HoneywellAltimeter = (HoneywellAltimeter) malloc(sizeof(struct honeywellaltimeter));
	if (NULL == this_HoneywellAltimeter)
		return NULL;

	this_HoneywellAltimeter->device.i2c_bus		= i2c_device->i2c_bus;
	this_HoneywellAltimeter->device.i2c_address = i2c_device->i2c_address;
	this_HoneywellAltimeter->device.i2c_flags	= i2c_device->i2c_flags;
	this_HoneywellAltimeter->device.i2c_delay	= i2c_device->i2c_delay;

	this_HoneywellAltimeter->pressureMin = pressureMin;
	this_HoneywellAltimeter->pressureMax = pressureMax;

	return this_HoneywellAltimeter;
}

cyg_uint32 HoneywellAltimeter_readPressure(HoneywellAltimeter this_HoneywellAltimeter){
	volatile cyg_uint8	buff[2] = {0,0};
	volatile cyg_uint32 	ret_count;
	cyg_i2c_device *Altimeter = &(this_HoneywellAltimeter->device);
	double pressureMin = this_HoneywellAltimeter-> pressureMin;
	double pressureMax = this_HoneywellAltimeter-> pressureMax;
	extern double airdata_bias[];
	
	cyg_i2c_transaction_begin( Altimeter );		
	ret_count = cyg_i2c_transaction_rx( Altimeter, true, (void *) buff, 2, true, true);
	cyg_i2c_transaction_end( Altimeter );
//	printf("Altimeter ret_count %d\n",ret_count);
	if(0 == ret_count)
		return ret_count;

	cyg_uint16 outputBinary = ( ((cyg_uint16) buff[0])<<8 ) + ((cyg_uint16) buff[1]);
	double absPercentage = ((double) outputBinary) / ((double) pow(2.0,14.0));
	double currentPressure = ( (absPercentage - 0.10)*(pressureMax-pressureMin) ) / 0.8 + pressureMin;
	currentPressure=currentPressure*PSI_TO_KPA;
	this_HoneywellAltimeter -> pressureReading = currentPressure;



	double currentAltitude = AIR_DATA_K1*(1-pow((currentPressure)/AIR_DATA_P0,AIR_DATA_K2)); //[ m ]
	this_HoneywellAltimeter -> altitudeReading = currentAltitude -airdata_bias[0];

	return ret_count;
}

double HoneywellAltimeter_getPressure(HoneywellAltimeter this_HoneywellAltimeter){
	return this_HoneywellAltimeter->pressureReading;
}
double HoneywellAltimeter_getAltitude(HoneywellAltimeter this_HoneywellAltimeter){
	return this_HoneywellAltimeter -> altitudeReading;
}
void delete_HoneywellAltimeter(HoneywellAltimeter this_HoneywellAltimeter){
	free (this_HoneywellAltimeter);
}
