/******************************************************************************
 * FILE: HoneywellPitot.c
 *
 * DESCRIPTION:
 * This is an Object Oriented C class that is able to read the data of 
 * HSC*RR*001PD*A3 Honeywell differential pressure sensor. It has two methods
 * readPressure which samples the pressure data, and getPressure which returns the
 * differential pressure from pressureMin psi to pressureMax psi
 *
 * CREATED: 06-28-10: Kyle Dobitz
 * University of Minnesota 
 * Aerospace Engineering and Mechanics 
 * Copyright 2011 Regents of the University of Minnesota. All rights reserved.
 *
 * $Id: HoneywellPitot.c 332 2011-04-14 18:43:14Z murch $
 ******************************************************************************/

#include <stdlib.h> 
#include <math.h>
#include <cyg/io/i2c_mpc5xxx.h>
#include "HoneywellPitot.h" 

struct honeywellpitot{
	cyg_i2c_device device;
	double pressureReading;
	double pressureMin;
	double pressureMax;
	double speedReading;
};

HoneywellPitot new_HoneywellPitot(cyg_i2c_device *i2c_device, double pressureMin, double pressureMax){
	
	HoneywellPitot this_HoneywellPitot = (HoneywellPitot) malloc(sizeof(struct honeywellpitot));
	if (NULL == this_HoneywellPitot)
		return NULL;

	this_HoneywellPitot->device.i2c_bus		= i2c_device->i2c_bus;
	this_HoneywellPitot->device.i2c_address = i2c_device->i2c_address;
	this_HoneywellPitot->device.i2c_flags	= i2c_device->i2c_flags;
	this_HoneywellPitot->device.i2c_delay	= i2c_device->i2c_delay;

	this_HoneywellPitot->pressureMin = pressureMin;
	this_HoneywellPitot->pressureMax = pressureMax;

	return this_HoneywellPitot;
}

cyg_uint32 HoneywellPitot_readPressure(HoneywellPitot this_HoneywellPitot){
	volatile cyg_uint8	buff[2] = {0,0};
	volatile cyg_uint32 	ret_count;
	cyg_i2c_device *pitot = &(this_HoneywellPitot->device);
	double pressureMin = this_HoneywellPitot->pressureMin;
	double pressureMax = this_HoneywellPitot->pressureMax;
	extern double airdata_bias[];

	cyg_i2c_transaction_begin( pitot );		
	ret_count = cyg_i2c_transaction_rx( pitot, true, (void *) buff, 2, true, true);
	cyg_i2c_transaction_end( pitot );
//	printf("\nPitot ret_count %d\n",ret_count);
	
	if(0 == ret_count)
		return ret_count;

	cyg_uint16 outputBinary = ( ((cyg_uint16) buff[0])<<8 ) + ((cyg_uint16) buff[1]);
	double diffPercentage = ((double) outputBinary) / ((double) pow(2.0,14.0));
	double currentPressure = ( (diffPercentage - 0.10)*(pressureMax-pressureMin) ) / 0.8 + pressureMin;
	currentPressure= currentPressure*PSI_TO_KPA;
	this_HoneywellPitot->pressureReading = currentPressure;



	double currentSpeed  = copysign(AIR_DATA_K3*sqrt(fabs(pow(fabs((currentPressure-airdata_bias[1])/AIR_DATA_P0 +1),AIR_DATA_K4)-1)),currentPressure); //remove pressure bias before computing speed
	this_HoneywellPitot->speedReading = currentSpeed;

	return ret_count;
}

double HoneywellPitot_getPressure(HoneywellPitot this_HoneywellPitot){
	return this_HoneywellPitot->pressureReading;
}
double HoneywellPitot_getSpeed(HoneywellPitot this_HoneywellPitot){
	return this_HoneywellPitot->speedReading;
}
void delete_HoneywellPitot(HoneywellPitot this_HoneywellPitot){
	free (this_HoneywellPitot);
}
